"use strict";

exports.compileTemplate = compileTemplate;
var defaultSettings = {
  reExpression: /{{\s*([\s\S]+?)\s*}}/g,
  reStatement: /<<\s*([\s\S]+?)\s*>>/g,
  contextName: '$' // Used to match unescaped characters in compiled string literals

};
var reSpecialChars = /['\n\r\u2028\u2029\\]/g; // Used to escape characters for inclusion in compiled string literals

var specialEscapes = {
  '\\': '\\',
  "'": "'",
  '\n': 'n',
  '\r': 'r',
  "\u2028": 'u2028',
  "\u2029": 'u2029'
};

function escapeSpecialChar(char) {
  return specialEscapes[char] ? "\\" + specialEscapes[char] : char;
}

function compileTemplate(string, settings) {
  if (typeof string !== 'string') {
    throw Error("Expected a template string, got " + string);
  }

  settings = patch(defaultSettings, settings);
  var codeBody = ''; // Compile the regexp to match each delimiter.

  var reDelimiters = RegExp([settings.reExpression.source, settings.reStatement.source].join('|') + "|$", 'g');
  var index = 0;
  string.replace(reDelimiters, function (match, expressionValue, statementValue, offset) {
    var content = string.slice(index, offset) // Escape the characters that can't be included in string literals
    .replace(reSpecialChars, escapeSpecialChar);
    if (content) codeBody += "__append('" + content + "')\n";
    if (statementValue) codeBody += statementValue + "\n";
    if (expressionValue) codeBody += "__append(" + expressionValue + ")\n";
    index = offset + match.length;
    return match;
  });
  var _settings = settings,
      contextName = _settings.contextName;
  return Function(['locals'], "'use strict'\n\nvar " + contextName + " = {}\nif (locals) for (var key in locals) " + contextName + "[key] = locals[key]\n\nvar __out = ''\nfunction __append(val) {if (val != null) __out += val}\n\n" + codeBody + "\nreturn __out");
}

function patch(left, right) {
  var out = {};
  if (isDict(left)) for (var key in left) {
    out[key] = left[key];
  }
  if (isDict(right)) for (var _key in right) {
    out[_key] = right[_key];
  }
  return out;
}

function isDict(value) {
  return isObject(value) && isPlainPrototype(Object.getPrototypeOf(value));
}

function isObject(value) {
  return value !== null && typeof value === 'object';
}

function isPlainPrototype(value) {
  return value === null || value === Object.prototype;
}